%--------------------------------------------------------------------------
% Copyright 2019-2020  Institute of Materials and Structures, Riga Technical University
%
% Permission is hereby granted, free of charge, to any person obtaining a
% copy of this software and associated documentation files (the "Software"),
% to deal in the Software without restriction, including without limitation
% the rights to use, copy, modify, merge, publish, distribute, sublicense,
% and/or sell copies of the Software, and to permit persons to whom the
% Software is furnished to do so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included
% in all copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
% OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
% ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
% OTHER DEALINGS IN THE SOFTWARE.
%--------------------------------------------------------------------------
%
% This code is a work in progress. Not intended for end-user applications.
%
% This code is to be used together with Polytec's code for accessing SVD files.
%

function process

SVDIdx = 1; % index for a package of SVD files

result = [];

createPlots = 1; % 0=no; 1=all except modes; 2=all

pathToSave = 'G:\Cache\';

%--------------------------------------------------------------------------

freqPlotBig = false;

MinPeakHeightDivisor = 30;
MinPeakProminenceDivisor = 400;
D = 0;
H = 0;
L = 1;
ver = 1;
PSV400 = false;
swapAxis = false;
remove50Hz = false;
removeFreq = [];
removeMicroPeaks = false;
noPeaksBefore = 0;
addToFilename = '';
material = '';
Pcr = 0;
EBL = 0;

%--------------------------------------------------------------------------

switch SVDIdx
    case 1
        pathToLoad = 'G:\Cache\D100H200L1_03\';
        MinPeakProminenceDivisor = 150;
        D = 100;
        H = 200;
        ver = 3;
        Pcr = 6.71;
        EBL = 3.12;
end

outputFilename = [pathToSave 'D' num2str(D) 'H' num2str(H) 'L' num2str(L) material '-0' num2str(ver) addToFilename];

%--------------------------------------------------------------------------

filelist = dir(pathToLoad);
filelist = {filelist.name};

NN = [];
for iFile1 = 1 : length(filelist)
    filename = filelist{iFile1};
    if (length(filename) < 6) || ~strcmp(filename((end-3):end), '.svd')
        NN(end+1) = Inf;
        continue;
    end
    if strcmp(filename((end-5):end), 'kN.svd')
        N = strsplit(filename, '_');
        N = N{end};
        NN(end+1) = str2num(N(1:(end-6))) * 1000;
    elseif strcmp(filename((end-4):end), 'N.svd')
        N = strsplit(filename, '_');
        N = N{end};
        NN(end+1) = str2num(N(1:(end-5)));
    else
        N = strsplit(filename, '_');
        N = N{end-1};
        NN(end+1) = str2num(N(1:(end-1)));
    end
end
[~, idx] = sort(NN);
filelist = filelist(idx);

dataN = [];
dataInfo = {};
counter = 0;
filenamesUsed = {};
for iFile1 = 1 : length(filelist)
    filename = filelist{iFile1};
    if (length(filename) < 6) || ~strcmp(filename((end-3):end), '.svd')
        continue;
    end
    
    if strcmp(filename((end-5):end), 'kN.svd')
        N = strsplit(filename, '_');
        N = N{end};
        N = str2num(N(1:(end-6))) * 1000;
    elseif strcmp(filename((end-4):end), 'N.svd')
        N = strsplit(filename, '_');
        N = N{end};
        N = str2num(N(1:(end-5)));
    else
        N = strsplit(filename, '_');
        N = N{end-1};
        N = str2num(N(1:(end-1)));
    end
    
    disp(filename);
    
    filename = [pathToLoad filename];
    filenamesUsed{end+1} = filename;
    
    if PSV400
        [freq, vibZ] = GetPointData(filename, 'FFT', 'Vib', 'Velocity', 'Magnitude', 0, 0);
    else
        [freq, vibZ] = GetPointData(filename, 'FFT', 'Vib Z', 'Velocity', 'Magnitude', 0, 0);
    end
    vibZ = vibZ * 1e6;
    
    [locsZ, peaksZ, loss, wmf] = processFile(freq, vibZ, N, pathToSave, MinPeakHeightDivisor, MinPeakProminenceDivisor, remove50Hz, removeMicroPeaks, noPeaksBefore, freqPlotBig, iFile1, createPlots, removeFreq);
    
    dataN = [dataN N];
    
    dataOneFile = {};
    for j = 1 : length(locsZ)
        loc = locsZ(j);
        idx = 0;
        for k = 1 : length(freq)
            if loc == freq(k)
                idx = k;
                break;
            end
        end
        if loss(j) < 0
            dataPeak.loss = NaN;
        else
            dataPeak.loss = loss(j);
        end
        dataPeak.idx = idx;
        dataPeak.freq = locsZ(j);
        dataPeak.peaksZ = peaksZ(j);
        dataPeak.mode = vibZ(:,idx);
        dataOneFile = [dataOneFile dataPeak];
    end
    
    counter = counter + 1;
    dataInfo{counter,1} = dataOneFile;
end

maxLen = 0;
for iFile1 = 1 : length(dataInfo)
    if length(dataInfo{iFile1}) > maxLen
        maxLen = length(dataInfo{iFile1});
    end
end
for iFile1 = 1 : length(dataInfo)
    if length(dataInfo{iFile1}) < maxLen
        dataInfo{iFile1} = [dataInfo{iFile1} cell(1,maxLen - length(dataInfo{iFile1}))];
    end
end

j = 0;
while j < maxLen
    j = j + 1;
    for iFile1 = 1 : (length(dataInfo) - 1)
        if isempty(dataInfo{iFile1}{j})
            continue;
        end
        for iFile2 = (iFile1 + 1) : length(dataInfo)
            if isempty(dataInfo{iFile2}{j})
                if (iFile2 == length(dataInfo))
                    iFile2 = -1;
                    break;
                end
            else
                break;
            end
        end
        
        if iFile2 < 1
            continue;
        end
        M1 = dataInfo{iFile1}{j}.mode;
        M2 = dataInfo{iFile2}{j}.mode;
        if dataInfo{iFile1}{j}.freq > dataInfo{iFile2}{j}.freq
            MAC = calcMAC(M1, M2);
        else
            MAC = -2;
        end
        if MAC == -2
            fprintf('Moving file3. %d, %d, %d, %f\n', j, iFile1, iFile2, MAC);
            if j >= 2
                for iFile3 = iFile2 : length(dataInfo)
                    dataInfo{iFile3} = [dataInfo{iFile3}(1:(j-1)) cell(1,1) dataInfo{iFile3}(j:end)];
                end
            else
                for iFile3 = iFile2 : length(dataInfo)
                    dataInfo{iFile3} = [cell(1,1) dataInfo{iFile3}];
                end
            end
            maxLen = maxLen + 1;
            for iFile3 = 1 : length(dataInfo)
                if length(dataInfo{iFile3}) < maxLen
                    dataInfo{iFile3} = [dataInfo{iFile3} cell(1,1)];
                end
            end
            j = 0;
            break;
        end
        
        MACleft = -1;
        if (j - 1 >= 1) && (~isempty(dataInfo{iFile2}{j-1})) && (dataInfo{iFile1}{j}.freq > dataInfo{iFile2}{j-1}.freq)
            M2left = dataInfo{iFile2}{j-1}.mode;
        	MACleft = calcMAC(M1, M2left);
            if (MACleft <= MAC)
                MACleft = -1;
            end
        end
        MACright = -1;
        if (j + 1 < maxLen) && (~isempty(dataInfo{iFile2}{j+1})) && (dataInfo{iFile1}{j}.freq > dataInfo{iFile2}{j+1}.freq)
            M2right = dataInfo{iFile2}{j+1}.mode;
        	MACright = calcMAC(M1, M2right);
            if (MACright <= MAC) || (MACright <= MACleft)
                MACright = -1;
            end
        end
        
        if MACleft > 0
            fprintf('Moving file2. %d, %d, %d, %f\n', j, iFile1, iFile2, MACleft);
            if j >= 2
                dataInfo{iFile2} = [dataInfo{iFile2}(1:(j-1)) cell(1,1) dataInfo{iFile2}(j:end)];
            else
                dataInfo{iFile2} = [cell(1,1) dataInfo{iFile2}];
            end
            maxLen = maxLen + 1;
            for iFile3 = 1 : length(dataInfo)
                if length(dataInfo{iFile3}) < maxLen
                    dataInfo{iFile3} = [dataInfo{iFile3} cell(1,1)];
                end
            end
            j = 0;
            break;
        end
        
        if MACright > 0
            fprintf('Moving file1. %d, %d, %d, %f\n', j, iFile1, iFile2, MACright);
            if j >= 2
                dataInfo{iFile1} = [dataInfo{iFile1}(1:(j-1)) cell(1,1) dataInfo{iFile1}(j:end)];
            else
                dataInfo{iFile1} = [cell(1,1) dataInfo{iFile1}];
            end
            maxLen = maxLen + 1;
            for iFile3 = 1 : length(dataInfo)
                if length(dataInfo{iFile3}) < maxLen
                    dataInfo{iFile3} = [dataInfo{iFile3} cell(1,1)];
                end
            end
            j = 0;
            break;
        end
    end
end

% -----------------------------------------------------
% Here additional corrections to the alignment can be done
%switch SVDIdx
%end
% -----------------------------------------------------

maxLen = 0;
for iFile1 = 1 : length(dataInfo)
    for j = length(dataInfo{iFile1}) : -1 : 1
        if ~isempty(dataInfo{iFile1}{j})
            if j > maxLen
                maxLen = j;
            end
            break;
        end
    end
end
for iFile1 = 1 : length(dataInfo)
    dataInfo{iFile1} = dataInfo{iFile1}(1:maxLen);
end

for j = length(dataInfo{iFile1}) : -1 : 1
    empty = true;
    for iFile1 = 1 : length(dataInfo)
        if ~isempty(dataInfo{iFile1}{j})
            empty = false;
            break;
        end
    end
    if empty
        maxLen = maxLen - 1;
        for iFile1 = 1 : length(dataInfo)
            dataInfo{iFile1}(j) = [];
        end
    end
end

for j = 1 : maxLen
    for iFile1 = 1 : length(dataInfo)
        if ~isempty(dataInfo{iFile1}{j})
            dataInfo{iFile1}{j}.MAC = 0;
            dataInfo{iFile1}{j}.MAC2 = 0;
        end
    end
end
for j = 1 : maxLen
    M1 = [];
    for iFile1 = 1 : length(dataInfo)
        if isempty(dataInfo{iFile1}{j})
            continue;
        end
        M2 = dataInfo{iFile1}{j}.mode;
        if iFile1 == 1
            M1 = M2;
        end
        if isempty(M1)
            continue;
        end
        dataInfo{iFile1}{j}.MAC = calcMAC(M1, M2);
    end
    for iFile1 = 1 : (length(dataInfo) - 1)
        if isempty(dataInfo{iFile1}{j}) || isempty(dataInfo{iFile1 + 1}{j})
            continue;
        end
        M1 = dataInfo{iFile1}{j}.mode;
        M2 = dataInfo{iFile1 + 1}{j}.mode;
        dataInfo{iFile1 + 1}{j}.MAC2 = calcMAC(M1, M2);
    end
end

for iFile1 = 1 : length(dataInfo)
    XYZ = GetXYZCoordinates(filenamesUsed{iFile1}, 0);
    for j = 1 : maxLen
        if isempty(dataInfo{iFile1}{j})
            continue;
        end
        [n, m] = saveMode(XYZ, dataInfo{iFile1}{j}.mode, dataN(iFile1), dataInfo{iFile1}{j}.freq, pathToSave, D, iFile1, j, createPlots, swapAxis);
        dataInfo{iFile1}{j}.n = n;
        dataInfo{iFile1}{j}.m = m;
    end
end

dataColumns = {'Load N'};
for j = 1 : maxLen
    dataColumns{end+1,:} = ['(n, m) [' num2str(j) ']'];
    dataColumns{end+1,:} = 'f, Hz';
    dataColumns{end+1,:} = ['z, ' char(181) 'm/s'];
    dataColumns{end+1,:} = '\x03B7';
    dataColumns{end+1,:} = 'MAC';
    dataColumns{end+1,:} = 'MAC2';
end
data = {};
for iFile1 = 1 : length(dataInfo)
    data1 = {dataN(iFile1)};
    for j = 1 : maxLen
        if isempty(dataInfo{iFile1}{j})
            data1 = [data1 NaN NaN NaN NaN NaN NaN];
        else
            nm = {['(' num2str(dataInfo{iFile1}{j}.n) ', ' num2str(dataInfo{iFile1}{j}.m) ')']};
            data1 = [data1 nm dataInfo{iFile1}{j}.freq dataInfo{iFile1}{j}.peaksZ dataInfo{iFile1}{j}.loss dataInfo{iFile1}{j}.MAC dataInfo{iFile1}{j}.MAC2];
        end
    end
    data(end+1,1:length(data1)) = data1;
end

ex = length(dataInfo) + 6; %excel index offset

for j = 1 : maxLen
    data(end+1,:) = num2cell(NaN(1,size(data,2)));
    data1 = {['Freq ' num2str(j)]};
    data(end+1,:) = [data1 num2cell(NaN(1,size(data,2) - size(data1,2)))];
    data1 = {'LBL' num2str(Pcr) 'EBL' num2str(EBL)};
    data(end+1,:) = [data1 num2cell(NaN(1,size(data,2) - size(data1,2)))];
    data1 = {'Load N' 'f, Hz' ...
             '(f/f0)^2' 'P/Pcr' '(f/f0)^4' 'P/Pcr' '1-(f/f0)^2' '(1-(P/Pcr))^2' ...
             '' 'Percent' 'P1' 'Dev' 'P2' 'Dev' 'P3' 'Dev'};
    data(end+1,:) = [data1 num2cell(NaN(1,size(data,2) - size(data1,2)))];
    
    n = length(dataInfo);
    x = NaN(n,3);
    y = NaN(n,3);
    for iFile1 = 1 : n
        if isempty(dataInfo{iFile1}{j})
            if iFile1 == 1
                break;
            else
                continue;
            end
        end
        x(iFile1,1) = (dataInfo{iFile1}{j}.freq / dataInfo{1}{j}.freq) ^ 2;
        y(iFile1,1) = dataN(iFile1) / 1000 / Pcr;
        
        x(iFile1,2) = (dataInfo{iFile1}{j}.freq / dataInfo{1}{j}.freq) ^ 4;
        y(iFile1,2) = dataN(iFile1) / 1000 / Pcr;
        
        x(iFile1,3) = 1 - (dataInfo{iFile1}{j}.freq / dataInfo{1}{j}.freq) ^ 2;
        y(iFile1,3) = (1 - dataN(iFile1) / 1000 / Pcr)^2;
    end
    
    iFile1From = 1;
    
    P = nan(3,n);
    MSE = nan(3,n);
    for iFile1 = 1 : n
        
        if length(iFile1From:iFile1) < 2
            P(:,iFile1) = NaN;
            MSE(:,iFile1) = NaN;
            continue;
        end
        
        for iModel = 1 : 2
            yy = y(iFile1From:iFile1,iModel);
            xx = x(iFile1From:iFile1,iModel);
            yy(isnan(xx)) = [];
            xx(isnan(xx)) = [];
            if length(xx) >= 2
                params = polyfit(yy, xx, 1);
                a = [params(2) params(1)];
                P(iModel,iFile1) = Pcr / (-a(1) / a(2));
                
                Yhat = [xx ones(length(xx),1)] * params';
                MSE(iModel,iFile1) = mean((yy - Yhat) .^ 2);
                
                if (iModel == 1) || (iModel == 2)
                    xxCurve = [0 1];
                    yyCurve = [a(1) a(1) + a(2)];
                    if (iModel == 1)
                        yName = '{\it f}^2';
                        titleStr = '';
                        filename = [outputFilename '_m1_' num2str(j)];
                    else
                        yName = '{\it f}^4';
                        titleStr = '';
                        filename = [outputFilename '_m2_' num2str(j)];
                    end
                    filename = [filename '_' num2str(iFile1)];
                    pred = P(iModel,iFile1);
                    annotate = { ...
                        ['LBL = ' num2str(Pcr) ' kN'], ...
                        ['EBL = ' num2str(EBL) ' kN'], ...
                        ['P_{VCT} = ' num2str(round(pred * 100) / 100) ' kN'], ...
                        ['Deviation = ' num2str(round((EBL - pred) / EBL * 1000) / 10) '%']};
                    if createPlots >= 1
                        plotVCT([yy xx], [xxCurve' yyCurve'], [], '{\it p}', yName, titleStr, annotate, filename);
                    end
                end
            else
                P(iModel,iFile1) = NaN;
                MSE(iModel,iFile1) = NaN;
            end
        end
        
        if length(iFile1From:iFile1) < 3
            P(3:n,iFile1) = NaN;
            MSE(3:n,iFile1) = NaN;
            continue;
        end
        
        for iModel = 3
            yy = y(iFile1From:iFile1,iModel);
            xx = x(iFile1From:iFile1,iModel);
            which = isnan(xx) | isnan(yy);
            yy(which) = [];
            xx(which) = [];
            if length(xx) < 3
                P(iModel,iFile1) = NaN;
                MSE(iModel,iFile1) = NaN;
                continue;
            end
            params = polyfit(xx, yy, 2);
            a = [params(3) params(2) params(1)];
            xMin = -a(2) / (2 * a(3));
            yMin = a(3) * xMin^2 + a(2) * xMin + a(1);
            if yMin < 0
                yMin = 0;
            end
            KDF = 1 - sqrt(yMin);
            P(iModel,iFile1) = Pcr * KDF;
            
            Yhat = [xx.^2 xx ones(length(xx),1)] * params';
            MSE(iModel,iFile1) = mean((yy - Yhat) .^ 2);
            
            if (iModel == 3)
                maxX = max([xMin xx']);
                xxCurve = 0:0.001:maxX;
                yyCurve = a(3).*xxCurve.^2 + a(2).*xxCurve + a(1);
                if (iModel == 3)
                    xName = '1 - {\it f}^2';
                    titleStr = '';
                    filename = [outputFilename '_m3_' num2str(j)];
                else
                    xName = '1 - {\it f}';
                    titleStr = '';
                    filename = [outputFilename '_m4_' num2str(j)];
                end
                filename = [filename '_' num2str(iFile1)];
                pred = P(iModel,iFile1);
                annotate = { ...
                    ['LBL = ' num2str(Pcr) ' kN'], ...
                    ['EBL = ' num2str(EBL) ' kN'], ...
                    ['P_{VCT} = ' num2str(round(pred * 100) / 100) ' kN'], ...
                    ['Deviation = ' num2str(round((EBL - pred) / EBL * 1000) / 10) '%']};
                if createPlots >= 1
                    plotVCT([xx yy], [xxCurve' yyCurve'], [xMin yMin], xName, '(1 - {\it p})^2', titleStr, annotate, filename);
                end
            end
        end
    end
    
        for iFile1 = 1 : n
            if isempty(dataInfo{iFile1}{j})
                if iFile1 == 1
                    for iFile2 = iFile1 : n
                        data1 = {dataN(iFile1)};
                        data(end+1,:) = [data1 num2cell(NaN(1,size(data,2) - size(data1,2)))];
                    end
                    break;
                else
                    data1 = {dataN(iFile1)};
                    data(end+1,:) = [data1 num2cell(NaN(1,size(data,2) - size(data1,2)))];
                    continue;
                end
            end
            data1 = {dataN(iFile1) dataInfo{iFile1}{j}.freq ...
                ['=(B' num2str(ex + iFile1 - 1) '/$B$' num2str(ex) ')^2'] ...
                ['=A' num2str(ex + iFile1 - 1) '/1000/$B$' num2str(ex - 2)] ...
                ['=(B' num2str(ex + iFile1 - 1) '/$B$' num2str(ex) ')^4'] ...
                ['=A' num2str(ex + iFile1 - 1) '/1000/$B$' num2str(ex - 2)] ...
                ['=1-(B' num2str(ex + iFile1 - 1) '/$B$' num2str(ex) ')^2'] ...
                ['=(1-A' num2str(ex + iFile1 - 1) '/1000/$B$' num2str(ex - 2) ')^2'] ...
                '' ...
                ['=A' num2str(ex + iFile1 - 1) '/1000/$D$' num2str(ex - 2)] ...
                num2str(P(1,iFile1)) ...
                ['=(($D$' num2str(ex - 2) '-Q' num2str(ex + iFile1 - 1) ')/$D$' num2str(ex - 2) ')*100'] ...
                num2str(P(2,iFile1)) ...
                ['=(($D$' num2str(ex - 2) '-S' num2str(ex + iFile1 - 1) ')/$D$' num2str(ex - 2) ')*100'] ...
                num2str(P(3,iFile1)) ...
                ['=(($D$' num2str(ex - 2) '-U' num2str(ex + iFile1 - 1) ')/$D$' num2str(ex - 2) ')*100']
                };
            data(end+1,:) = [data1 num2cell(NaN(1,size(data,2) - size(data1,2)))];
        end
        
    data1 = {NaN NaN 'P1' num2str(P(1,n)) 'P2' num2str(P(2,n)) 'P3' num2str(P(3,n))};
    data(end+1,:) = [data1 num2cell(NaN(1,size(data,2) - size(data1,2)))];
    data1 = {NaN NaN ...
        'Deviation' ['=(($D$' num2str(ex - 2) '-D' num2str(ex + n) ')/$D$' num2str(ex - 2) ')*100'] ...
        'Deviation' ['=(($D$' num2str(ex - 2) '-F' num2str(ex + n) ')/$D$' num2str(ex - 2) ')*100'] ...
        'Deviation' ['=(($D$' num2str(ex - 2) '-H' num2str(ex + n) ')/$D$' num2str(ex - 2) ')*100']};
    data(end+1,:) = [data1 num2cell(NaN(1,size(data,2) - size(data1,2)))];
    
    ex = ex + n + 6;
    
    if j == maxLen
        for iFile1 = 3 : n
            result = [result; dataN(iFile1) dataInfo{iFile1}{j}.freq dataN(iFile1)/1000/EBL ...
                P(3,iFile1) (EBL-P(3,iFile1))/EBL*100 P(4,iFile1) (EBL-P(4,iFile1))/EBL*100];
        end
    end
end

writecsv([outputFilename '.csv'], data, dataColumns, false);
end

%--------------------------------------------------------------------------

function plotVCT(points, curve, predictionPoint, xName, yName, titleStr, annotate, outputFilename)
set(0, 'Units', 'pixels');
screenSize = get(0, 'screensize');
winSize = [600 400];
f = figure('Units', 'pixels', 'Position', [screenSize(3)/2-winSize(1)/2 screenSize(4)-winSize(2)-92 winSize]);
hold all;
plot(points(:,1), points(:,2), '.k', 'Linewidth', 2, 'MarkerSize', 20);
plot(curve(:,1), curve(:,2), '-', 'Linewidth', 2);
if ~isempty(predictionPoint)
    plot([0 predictionPoint(1,1)], [predictionPoint(1,2) predictionPoint(1,2)], '--r', 'Linewidth', 2, 'Linewidth', 2);
end

if ~isempty(predictionPoint)
    ylim([0 1]);
else
    yl = ylim;
    if yl(1) >= 1
        yl(1) = 0;
    end
    yl(2) = 1;
    ylim(yl);
end
box on;
grid on;
xlabel(xName);
ylabel(yName);
title(titleStr);

if isempty(predictionPoint)
    l = legend('Experiment', 'Best fit');
else
    l = legend('Experiment', 'Best fit', ['\xi^2 = ' num2str(round(predictionPoint(1,2) * 1000) / 1000)]);
end

pos = l.Position;
pos(1) = pos(1) - 0.03;
pos(3) = pos(3) + 0.03;
if isempty(predictionPoint)
    pos(2) = 0.55;
    pos(4) = pos(4) * 1.4 * 2 * 0.8;
else
    pos(2) = 0.5;
    pos(4) = pos(4) * 0.9 * 2 * 0.8;
end
annotation('textbox', pos, 'String', annotate, 'BackgroundColor', 'w', 'LineWidth', 0.25);

saveImage(f, [outputFilename '.png'], true);
end

%--------------------------------------------------------------------------

function MAC = calcMAC(M1, M2)
    if length(M1) ~= length(M2)
        MAC = NaN;
        return;
    end
    MAC = (abs(M1' * M2))^2 / ((M1' * M1) * (M2' * M2));
end

%--------------------------------------------------------------------------

function [locsZ, peaksZ, loss, weightedMeanFreq] = processFile(freq, vibZ, N, pathToSave, MinPeakHeightDivisor, MinPeakProminenceDivisor, remove50Hz, removeMicroPeaks, noPeaksBefore, big, fileID, createPlots, removeFreq)
avgZ = mean(vibZ);
[peaksZ, locsZ, W, P] = findpeaks(avgZ, freq, 'MinPeakHeight', max(avgZ) / MinPeakHeightDivisor, 'MinPeakProminence', max(avgZ) / MinPeakProminenceDivisor, 'MinPeakDistance', 1);

weightedSum = 0;
weightedDivisor = 0;
for i = 1 : length(freq)
    weightedSum = weightedSum + freq(i) * avgZ(i);
    weightedDivisor = weightedDivisor + avgZ(i);
end
weightedMeanFreq = weightedSum / weightedDivisor;

for iz = length(peaksZ) : -1 : 1
    loc = locsZ(iz);
    idx = 0;
    for i = 1 : length(freq)
        if loc == freq(i)
            idx = i;
            break;
        end
    end
    ok = true;
    if removeMicroPeaks
        ok = (idx - 3 >= 1) && (idx + 3 <= length(avgZ)) && ...
             avgZ(idx) > avgZ(idx - 1) && avgZ(idx) > avgZ(idx - 2) && avgZ(idx) > avgZ(idx - 3) && ...
             avgZ(idx) > avgZ(idx + 1) && avgZ(idx) > avgZ(idx + 2) && avgZ(idx) > avgZ(idx + 3);
    end
    if remove50Hz && (floor(freq(idx) / 50) == freq(idx) / 50) && ((W(iz) < 4) || (P(iz) < max(avgZ) / MinPeakProminenceDivisor * 1.5))
        ok = false;
    end
    if (~isempty(removeFreq)) && (abs(freq(idx) - removeFreq) <= 2)
        ok = false;
    end
    if freq(idx) < noPeaksBefore
        ok = false;
    end
    if ~ok
        peaksZ(iz) = [];
        locsZ(iz) = [];
        W(iz) = [];
        P(iz) = [];
    end
end

if createPlots >= 1
    set(0, 'Units', 'pixels');
    screenSize = get(0, 'screensize');
    if big
        winSize = [1200 600];
    else
        winSize = [500 300];
    end
    f = figure('Units', 'pixels', 'Position', [screenSize(3)/2-winSize(1)/2 screenSize(4)-winSize(2)-92 winSize]);
    hold all;
    plot(freq, avgZ);
    plot(locsZ, peaksZ, 'ok');

    box on;
    grid on;
    if big
        set(gca, 'FontSize', 14);
        xlabel('Freqeuncy (Hz)', 'FontSize', 16);
        ylabel('Magnitude (m/s)', 'FontSize', 16);
    else
        xlabel('Freqeuncy (Hz)');
        ylabel('Magnitude (m/s)');
        title('Frequency response');
    end
end

fromF = zeros(size(locsZ));
toF = zeros(size(locsZ));
loss = zeros(size(locsZ)) - 1;
for iz = 1 : length(locsZ)
    loc = locsZ(iz);
    idx = 0;
    for i = 1 : length(freq)
        if loc == freq(i)
            idx = i;
            break;
        end
    end
    h = peaksZ(iz) / sqrt(2);
    i1 = 0;
    for i = (idx - 1) : -1 : 1
        if avgZ(i) < h
            i1 = i + 1;
            break;
        end
    end
    i2 = 0;
    for i = (idx + 1) : length(avgZ)
        if avgZ(i) < h
            i2 = i - 1;
            break;
        end
    end
    if (i1 > 0) && (i2 > 0)
        fromF(iz) = freq(i1);
        toF(iz) = freq(i2);
    else
        continue;
    end
    
    bad = false;
    for iz2 = 1 : length(locsZ)
        if iz == iz2
            continue;
        end
        loc = locsZ(iz2);
        idx = 0;
        for i = 1 : length(freq)
            if loc == freq(i)
                idx = i;
                break;
            end
        end
        if ((fromF(iz) <= freq(idx)) && (toF(iz) >= freq(idx)))
            bad = true;
            break;
        end
    end
    if bad
        continue;
    end
    
    loss(iz) = (toF(iz) - fromF(iz)) / locsZ(iz);
end

if createPlots >= 1
    saveImage(f, [pathToSave 'diag_' num2str(fileID) '.png'], true);
end
end

%--------------------------------------------------------------------------

function [n, m] = saveMode(XYZ, vibZ, N, freq, pathToSave, D, fileID, freqID, createPlots, swapAxis)
coordX = 1;
if swapAxis
    coordY = 3;
else
    coordY = 2;
end
data = [XYZ(:,coordX), XYZ(:,coordY), vibZ];

curr = XYZ(1,coordY);
for i = 2 : size(XYZ,1)
    if XYZ(i,coordY) > curr
        rows = i - 1;
        cols = floor(size(XYZ,1) / rows);
        if rows * cols ~= size(XYZ,1)
            error('Was not able to derive number of rows and cols for the grid.');
        end
        break;
    end
    curr = XYZ(i,coordY);
end

width = max(data(:,1)) - min(data(:,1));
middleIdxTop = (floor(cols / 2) * rows) + 1;
middleIdxBottom = (floor(cols / 2) * rows) + rows;
height = data(middleIdxTop,2) - data(middleIdxBottom,2);
[xq, yq] = meshgrid(linspace(0,width,cols), linspace(0,height,rows));
zq = vec2mat(vibZ, rows)';

if createPlots >= 2
    f = figure;
    surf(xq, yq, zq);
    shading interp;
    view(2);
    grid off;
    axis off;
    axis equal;
    set(gca, 'XTickLabels', [], 'YTickLabels', [], 'XTick', [], 'YTick', []);
    colormap jet;

    export_fig([pathToSave 'mode_' num2str(freqID) '_' num2str(fileID) '.png'], '-png', '-transparent');
    close(f);
end

n = 0;
m = 0;
end
